(* ::Package:: *)

Needs["xAct`xTensor`"];
Needs["xAct`xCoba`"];

$PrePrint = ScreenDollarIndices;


(********** 3-manifold and time ************)
DefManifold[M3,3,{i,i2,j,j2,k,l,m,m1,m2,n,p}];
(*DefBasis[cart,TangentM3,{1,2,3},BasisColor->RGBColor[0,0,1]];*)

(********** metric and related stuff **********)
DefMetric[1, metrich[-i, -j], D3, {"|", "D"}, PrintAs -> "h", WeightedWithBasis -> AIndex];

(***** the direction to replace the derivatives *********)
DefTensor[s[-i], {M3}, PrintAs->"s"];
(***** orthonormal spatial basis ****)
DefTensor[V1[-i], {M3}, PrintAs->"V1"];
DefTensor[V2[-i], {M3}, PrintAs->"V2"];

AutomaticRules[s, MakeRule[{s[-i]s[i],1}, MetricOn->All]];
AutomaticRules[s, MakeRule[{metrich[-i,-j] s[i] s[j], 1}, MetricOn->All]];
AutomaticRules[V1, MakeRule[{V1[-i]V1[i],1}, MetricOn->All]];
AutomaticRules[V1, MakeRule[{metrich[-i,-j] V1[i] V1[j], 1}, MetricOn->All]];
AutomaticRules[V2, MakeRule[{V2[-i]V2[i],1}, MetricOn->All]];
AutomaticRules[V2, MakeRule[{metrich[-i,-j] V2[i] V2[j], 1}, MetricOn->All]];
AutomaticRules[s, MakeRule[{s[-i]V1[i],0}, MetricOn->All]];
AutomaticRules[s, MakeRule[{metrich[-i,-j] s[i] V1[j], 0}, MetricOn->All]];
AutomaticRules[s, MakeRule[{s[-i]V2[i],0}, MetricOn->All]];
AutomaticRules[s, MakeRule[{metrich[-i,-j] s[i] V2[j], 0}, MetricOn->All]];
AutomaticRules[V1, MakeRule[{V1[-i]V2[i],0}, MetricOn->All]];
AutomaticRules[V1, MakeRule[{metrich[-i,-j] V1[i] V2[j], 0}, MetricOn->All]];



(******* ansatz candidate *******)
DefTensor[H11[i, j], {M3}, Symmetric[{1,2}]];
DefTensor[H22[i, j, k, l], {M3}, GenSet[Images[{2, 1, 4, 3}]]];
DefTensor[H14[i, l], {M3}];
DefTensor[H23[i, k], {M3}];
DefTensor[H33[], {M3}];
DefTensor[H44[k, l], {M3}, Symmetric[{1,2}]];

ansatzCandUL[i_, j_, k_, l_] := {{H11[i,j], 0},{0, H22[i,j,k,l]}};
ansatzCandUR[i_, k_, l_] := {{0, H14[i,l]}, {H23[i,k], 0}};
ansatzCandLL[j_, l_, k_] := {{0, H23[j,k]}, {H14[j,l], 0}};
ansatzCandLR[k_, l_] := {{H33[], 0}, {0,H44[k,l]}};

HMaxwell[i_,j_,l_,k_]:=Evaluate@ArrayFlatten[{{ansatzCandUL[i, j, k, l], ansatzCandUR[i, k, l]},
                       {ansatzCandLL[j, k, l], ansatzCandLR[k, l]}}];
DefConstantSymbol[H111];
DefConstantSymbol[H221];
DefConstantSymbol[H222];
DefConstantSymbol[A221];
DefConstantSymbol[H141];
DefConstantSymbol[H231];
DefConstantSymbol[H331];
DefConstantSymbol[H441];

IndexSet[H11[i_,j_], H111 metrich[i,j]];
IndexSet[H22[i_,j_,k_,l_],
         H222 (metrich[i,k] metrich[j,l] + metrich[i,l] metrich[j,k]) +
         H221 metrich[i,j]metrich[k,l] +
         A221 (metrich[i,k] metrich[j,l] - metrich[i,l] metrich[j,k])];
IndexSet[H14[i_,k_], H141 metrich[i,k]];
IndexSet[H23[i_,l_], H231 metrich[i,l]];
IndexSet[H33[], H331];
IndexSet[H44[k_,l_], H441 metrich[k,l]]; 


(******* Principal Part *******)

DefConstantSymbol[\[Mu]];

AMaxwell[p_,-i_,-k_,j_,l_]:=
{{0, 0, delta[p,-i], 0},
{0, 0, 0, delta[p,-i] delta[l,-k]},
{\[Mu] metrich[p,j], 0, 0, 0},
{0, metrich[p,j]delta[l,-k],
(1/\[Mu]-1)delta[p,-k], 0}};



(******* S matrix *******)
S[i_]:={{s[i],0,0,0},{0,s[i],0,0},{0,0,1,0},{0,0,0,1}};


(*** auxiliary function to decompose a matrix ***)
TensorDot[v_List, M_?MatrixQ, w_List] :=
Module[{i, j},
       ArrayFlatten[Table[M[[i, j]] Transpose[{v[[i]]}].{w[[j]]},
                          {i, Length[v]}, {j, Length[w]}]]];

HBasis :=
TensorDot[
	{ {s[-i], V1[-i], V2[-i]},
	  {s[-i] s[-l], V1[-i] s[-l], V2[-i] s[-l],
	   s[-i] V1[-l], V1[-i] V1[-l], V2[-i] V1[-l],
	   s[-i] V2[-l], V1[-i] V2[-l], V2[-i] V2[-l]} ,
      {1},
      {s[-l], V1[-l], V2[-l]}
           },
          HMaxwell[i,j,k,l],
          { {s[-j], V1[-j], V2[-j]},
		  {s[-j] s[-k], V1[-j] s[-k], V2[-j] s[-k],
	       s[-j] V1[-k], V1[-j] V1[-k], V2[-j] V1[-k],
	       s[-j] V2[-k], V1[-j] V2[-k], V2[-j] V2[-k]},
          {1},
          {s[-k], V1[-k], V2[-k]}
          }
] //ContractMetric // Simplification;
