(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     19273,        599]
NotebookOptionsPosition[     17666,        538]
NotebookOutlinePosition[     18046,        555]
CellTagsIndexPosition[     18003,        552]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell["\<\
Strong hyperbolicity analysis for free evolution formulation of the Maxwell \
equations for NR/HEP2 
David Hilditch\
\>", "Subtitle",
 CellChangeTimes->{{3.570347671721697*^9, 3.570347721730142*^9}, 
   3.570347780388916*^9, {3.577465095128714*^9, 3.577465098675465*^9}}],

Cell[CellGroupData[{

Cell["Define state vectors", "Text",
 CellChangeTimes->{{3.570347726586381*^9, 3.570347738163884*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"uGauge", "=", 
   RowBox[{"{", 
    RowBox[{"p2sPsi", ",", "psPhi"}], "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.5703477408437777`*^9, 3.570347745166952*^9}, {
  3.5703477871828737`*^9, 3.570347815590317*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"uConstraint", "=", 
   RowBox[{"{", 
    RowBox[{"Z", ",", "Es"}], "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.570347751359498*^9, 3.57034775233433*^9}, {
  3.570347825607745*^9, 3.5703478398386173`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"uPhysicalA", "=", 
   RowBox[{"{", 
    RowBox[{"psAA", ",", "EA"}], "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.570347756055327*^9, 3.570347772159181*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Principal Symbols", "Text",
 CellChangeTimes->{{3.570347849461033*^9, 3.570347855231175*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"PsGauge", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"\[Beta]s", ",", 
       RowBox[{"-", "1"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "\[Mu]\[CapitalPhi]"}], " ", 
        SuperscriptBox["\[Alpha]", "2"]}], ",", "\[Beta]s"}], "}"}]}], 
    "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.5703478687508698`*^9, 3.570347897623269*^9}, {
  3.5703480022252903`*^9, 3.570348021228104*^9}, {3.571775057706025*^9, 
  3.571775059650484*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"PsConstraint", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"\[Beta]s", ",", "\[Alpha]"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"\[Alpha]", ",", "\[Beta]s"}], "}"}]}], "}"}]}], ";"}]], "Input",\

 CellChangeTimes->{{3.57034788417417*^9, 3.570347917057994*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"PsPhysical", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"\[Beta]s", ",", 
       RowBox[{"-", "\[Alpha]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "\[Alpha]"}], ",", "\[Beta]s"}], "}"}]}], "}"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.570347887278488*^9, 3.570347925674036*^9}, {
  3.572029510925816*^9, 3.572029512451233*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Define analyze function to avoid repetition", "Text",
 CellChangeTimes->{{3.5703485059978123`*^9, 3.570348518515266*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Analyze", "=", 
   RowBox[{"Function", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"P", ",", "u"}], "}"}], ",", 
     RowBox[{"Prepend", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"P", "//", "Transpose"}], "//", "Eigenvalues"}], "//", 
           "MatrixForm"}], ",", 
          RowBox[{
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{"P", "//", "Transpose"}], "//", "Eigenvectors"}], ")"}],
             ".", "u"}], "//", "MatrixForm"}]}], "}"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
        "\"\<Speeds\>\"", ",", " ", "\"\<Characteristic variables\>\""}], 
        "}"}]}], "]"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.570348393150464*^9, 3.5703484441827602`*^9}, {
  3.570348475700938*^9, 3.570348478259569*^9}, {3.570349104165739*^9, 
  3.570349194789421*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Hyperbolicity Analysis", "Text",
 CellChangeTimes->{{3.57034926643145*^9, 3.570349275145296*^9}, {
  3.5703493308575163`*^9, 3.5703493408358793`*^9}}],

Cell["Gauge Block", "Text",
 CellChangeTimes->{{3.5703483011340733`*^9, 3.570348312137129*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"PsGauge", "//", "Eigensystem"}], "//", "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.57034803332554*^9, 3.57034808838203*^9}, {
  3.570349696787695*^9, 3.5703496987700853`*^9}}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"\[Beta]s", "-", 
       RowBox[{"\[Alpha]", " ", 
        SqrtBox["\[Mu]\[CapitalPhi]"]}]}], 
      RowBox[{"\[Beta]s", "+", 
       RowBox[{"\[Alpha]", " ", 
        SqrtBox["\[Mu]\[CapitalPhi]"]}]}]},
     {
      RowBox[{"{", 
       RowBox[{
        FractionBox["1", 
         RowBox[{"\[Alpha]", " ", 
          SqrtBox["\[Mu]\[CapitalPhi]"]}]], ",", "1"}], "}"}], 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", 
         FractionBox["1", 
          RowBox[{"\[Alpha]", " ", 
           SqrtBox["\[Mu]\[CapitalPhi]"]}]]}], ",", "1"}], "}"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.5703480383638906`*^9, {3.570348084168087*^9, 3.570348088643024*^9}, 
   3.5703482648950663`*^9, 3.570349221857031*^9, 3.570349280826048*^9, 
   3.572029521361927*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Analyze", "[", 
   RowBox[{"PsGauge", ",", "uGauge"}], "]"}], "//", "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.5703481176526012`*^9, 3.570348235826413*^9}, {
  3.57034854800202*^9, 3.570348575614801*^9}, {3.570349154621828*^9, 
  3.570349157173152*^9}}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"\<\"Speeds\"\>", "\<\"Characteristic variables\"\>"},
     {
      TagBox[
       RowBox[{"(", "\[NoBreak]", 
        TagBox[GridBox[{
           {
            RowBox[{"\[Beta]s", "-", 
             RowBox[{"\[Alpha]", " ", 
              SqrtBox["\[Mu]\[CapitalPhi]"]}]}]},
           {
            RowBox[{"\[Beta]s", "+", 
             RowBox[{"\[Alpha]", " ", 
              SqrtBox["\[Mu]\[CapitalPhi]"]}]}]}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column], "\[NoBreak]", ")"}],
       Function[BoxForm`e$, 
        MatrixForm[BoxForm`e$]]], 
      TagBox[
       RowBox[{"(", "\[NoBreak]", 
        TagBox[GridBox[{
           {
            RowBox[{"psPhi", "+", 
             RowBox[{"p2sPsi", " ", "\[Alpha]", " ", 
              SqrtBox["\[Mu]\[CapitalPhi]"]}]}]},
           {
            RowBox[{"psPhi", "-", 
             RowBox[{"p2sPsi", " ", "\[Alpha]", " ", 
              SqrtBox["\[Mu]\[CapitalPhi]"]}]}]}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column], "\[NoBreak]", ")"}],
       Function[BoxForm`e$, 
        MatrixForm[BoxForm`e$]]]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{{3.570348236207095*^9, 3.5703482655108643`*^9}, {
   3.5703485670864*^9, 3.570348575956266*^9}, {3.5703491526141977`*^9, 
   3.57034922271887*^9}, 3.570349280884301*^9, 3.572029521540578*^9}]
}, Open  ]],

Cell["Constraint Block", "Text",
 CellChangeTimes->{{3.5703483011340733`*^9, 3.57034832694545*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"PsPhysical", "//", "Eigensystem"}], "//", "MatrixForm"}]], "Input",\

 CellChangeTimes->{{3.57034803332554*^9, 3.57034808838203*^9}, {
  3.570348360967115*^9, 3.570348362389408*^9}}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{
       RowBox[{"-", "\[Alpha]"}], "+", "\[Beta]s"}], 
      RowBox[{"\[Alpha]", "+", "\[Beta]s"}]},
     {
      RowBox[{"{", 
       RowBox[{"1", ",", "1"}], "}"}], 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", "1"}], ",", "1"}], "}"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.5703480383638906`*^9, {3.570348084168087*^9, 3.570348088643024*^9}, 
   3.5703482648950663`*^9, 3.570348363164302*^9, 3.570348611484461*^9, 
   3.5703492809359207`*^9, 3.572029521569467*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Analyze", "[", 
   RowBox[{"PsConstraint", ",", "uConstraint"}], "]"}], "//", 
  "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.5703481176526012`*^9, 3.570348235826413*^9}, {
   3.5703483385142803`*^9, 3.570348355569676*^9}, {3.570348496810958*^9, 
   3.570348498176108*^9}, {3.5703486001917458`*^9, 3.570348608885644*^9}, 
   3.5703492855599422`*^9}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"\<\"Speeds\"\>", "\<\"Characteristic variables\"\>"},
     {
      TagBox[
       RowBox[{"(", "\[NoBreak]", 
        TagBox[GridBox[{
           {
            RowBox[{
             RowBox[{"-", "\[Alpha]"}], "+", "\[Beta]s"}]},
           {
            RowBox[{"\[Alpha]", "+", "\[Beta]s"}]}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column], "\[NoBreak]", ")"}],
       Function[BoxForm`e$, 
        MatrixForm[BoxForm`e$]]], 
      TagBox[
       RowBox[{"(", "\[NoBreak]", 
        TagBox[GridBox[{
           {
            RowBox[{"Es", "-", "Z"}]},
           {
            RowBox[{"Es", "+", "Z"}]}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column], "\[NoBreak]", ")"}],
       Function[BoxForm`e$, 
        MatrixForm[BoxForm`e$]]]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{{3.570348236207095*^9, 3.5703482655108643`*^9}, 
   3.570348371317389*^9, 3.570348498948452*^9, {3.570348609831973*^9, 
   3.57034861192074*^9}, {3.57034928098875*^9, 3.5703492860255423`*^9}, 
   3.572029521586796*^9}]
}, Open  ]],

Cell["Physical Block", "Text",
 CellChangeTimes->{{3.5703483011340733`*^9, 3.5703483320567636`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"PsGauge", "//", "Eigensystem"}], "//", "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.57034803332554*^9, 3.57034808838203*^9}}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"\[Beta]s", "-", 
       RowBox[{"\[Alpha]", " ", 
        SqrtBox["\[Mu]\[CapitalPhi]"]}]}], 
      RowBox[{"\[Beta]s", "+", 
       RowBox[{"\[Alpha]", " ", 
        SqrtBox["\[Mu]\[CapitalPhi]"]}]}]},
     {
      RowBox[{"{", 
       RowBox[{
        FractionBox["1", 
         RowBox[{"\[Alpha]", " ", 
          SqrtBox["\[Mu]\[CapitalPhi]"]}]], ",", "1"}], "}"}], 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", 
         FractionBox["1", 
          RowBox[{"\[Alpha]", " ", 
           SqrtBox["\[Mu]\[CapitalPhi]"]}]]}], ",", "1"}], "}"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.5703480383638906`*^9, {3.570348084168087*^9, 3.570348088643024*^9}, 
   3.5703482648950663`*^9, 3.570349281035904*^9, 3.572029521607551*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Analyze", "[", 
   RowBox[{"PsPhysical", ",", "uPhysicalA"}], "]"}], "//", 
  "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.5703493050941772`*^9, 3.5703493128046494`*^9}}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"\<\"Speeds\"\>", "\<\"Characteristic variables\"\>"},
     {
      TagBox[
       RowBox[{"(", "\[NoBreak]", 
        TagBox[GridBox[{
           {
            RowBox[{
             RowBox[{"-", "\[Alpha]"}], "+", "\[Beta]s"}]},
           {
            RowBox[{"\[Alpha]", "+", "\[Beta]s"}]}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column], "\[NoBreak]", ")"}],
       Function[BoxForm`e$, 
        MatrixForm[BoxForm`e$]]], 
      TagBox[
       RowBox[{"(", "\[NoBreak]", 
        TagBox[GridBox[{
           {
            RowBox[{"EA", "+", "psAA"}]},
           {
            RowBox[{"EA", "-", "psAA"}]}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column], "\[NoBreak]", ")"}],
       Function[BoxForm`e$, 
        MatrixForm[BoxForm`e$]]]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.570349313186737*^9, 3.572029521645833*^9}]
}, Open  ]]
}, Open  ]]
},
CellGrouping->Manual,
WindowSize->{740, 798},
WindowMargins->{{Automatic, 238}, {Automatic, 0}},
ShowSelection->True,
FrontEndVersion->"8.0 for Linux x86 (64-bit) (February 23, 2011)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[557, 20, 281, 6, 120, "Subtitle"],
Cell[CellGroupData[{
Cell[863, 30, 102, 1, 30, "Text"],
Cell[968, 33, 249, 6, 30, "Input"],
Cell[1220, 41, 243, 6, 30, "Input"],
Cell[1466, 49, 195, 5, 30, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[1698, 59, 99, 1, 27, "Text"],
Cell[1800, 62, 544, 16, 34, "Input"],
Cell[2347, 80, 328, 10, 30, "Input"],
Cell[2678, 92, 423, 13, 30, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[3138, 110, 127, 1, 30, "Text"],
Cell[3268, 113, 975, 28, 88, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[4280, 146, 156, 2, 27, "Text"],
Cell[4439, 150, 95, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[4559, 155, 214, 4, 30, "Input"],
Cell[4776, 161, 1323, 38, 65, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[6136, 204, 293, 6, 30, "Input"],
Cell[6432, 212, 2762, 72, 73, "Output"]
}, Open  ]],
Cell[9209, 287, 99, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[9333, 292, 217, 5, 30, "Input"],
Cell[9553, 299, 1039, 29, 45, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[10629, 333, 387, 8, 30, "Input"],
Cell[11019, 343, 2468, 66, 60, "Output"]
}, Open  ]],
Cell[13502, 412, 100, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[13627, 417, 163, 3, 30, "Input"],
Cell[13793, 422, 1297, 37, 65, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[15127, 464, 208, 5, 30, "Input"],
Cell[15338, 471, 2300, 63, 73, "Output"]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
